/* r8180_if.h: Part of Linux wireless lan driver for RTL8180. */
/*
        Written 2002-2003 by ShuChen.

        This file is distributed according to the terms of
        the GNU General Public License (GPL).

        The original author may be reached as shuchen@realtek.com.tw

		Versions:
		1.0 - first release
		1.1 - second release
		1.2 - 2003-06-10
*/

#ifndef _INCLUDE_RTL8180_IF_H_
#define _INCLUDE_RTL8180_IF_H_


#include "r8180_type.h"

#define rtl_memcpy          memcpy
#define rtl_memcmp          memcmp
#define rtl_memset          memset

#ifdef WITHOUT_LINUX_KERNEL
	#define rtl_udelay					_udelay
	#define rtl_mdelay					_mdelay
	extern void _udelay(unsigned long usec);
	extern void _mdelay(unsigned long msec);
#else
	#define rtl_udelay					udelay
	#define rtl_mdelay					mdelay
#endif    // WITOUT_LINUX_KERNEL

#define rtl_atomic_inc(ptr_atomic_t)	atomic_inc	(ptr_atomic_t)
#define rtl_atomic_dec(ptr_atomic_t)	atomic_dec(ptr_atomic_t)
#define rtl_atomic_add(i,ptr_atomic_t)	atomic_add(i,ptr_atomic_t)
#define rtl_atomic_sub(i,ptr_atomic_t)	atomic_sub(i,ptr_atomic_t)
#define rtl_atomic_read(ptr_atomic_t)	atomic_read(ptr_atomic_t)
#define rtl_atomic_set(ptr_atomic_t,i)	atomic_set(ptr_atomic_t,i)

#define rtl_skb_atomic_inc(ptr_atomic_t)	atomic_inc(ptr_atomic_t)
#define rtl_skb_atomic_dec(ptr_atomic_t)	atomic_dec(ptr_atomic_t)
#define rtl_skb_atomic_read(ptr_atomic_t)	atomic_read(ptr_atomic_t)
#define rtl_skb_atomic_set(ptr_atomic_t,i)	atomic_set(ptr_atomic_t,i)


//==========================================================================================
// rtl_alloc_wlandev()
//    based on alloc_etherdev() - Allocates and sets up an ethernet device.
//    @sizeof_priv: Size of additional driver-private structure to be allocated for this ethernet device
//    Fill in the fields of the device structure with ethernet-generic values.
//    Basically does everything except registering the device.
//    Constructs a new net device, complete with a private data area of
//    size @sizeof_priv.  A 32-byte (not bit) alignment is enforced for
//    this private data area.
//==========================================================================================
struct net_device *rtl_alloc_wlandev(struct net_device *dev, int sizeof_priv);

//==========================================================================================
// rtl_register_netdev()
//     Register the net device once everything initializes.
//==========================================================================================
int rtl_register_netdev(struct net_device *dev);

//==========================================================================================
// rtl_unregister_netdev()
//     Unregister the net device once everything initializes.
//==========================================================================================
void rtl_unregister_netdev(struct net_device *dev);

//==========================================================================================
struct sk_buff *rtl_dev_alloc_skb(unsigned int length);
void rtl_skb_reserve(struct sk_buff *skb, unsigned int len);
void rtl_kfree_skb(struct sk_buff *skb);


extern int rtl_malloc_cnt;

unsigned char *rtl_malloc(int size, int flags);
void rtl_free( unsigned char *buf);

int rtl_netif_running(struct net_device *dev);
int rtl_netif_queue_stopped(struct net_device *dev);
void rtl_netif_wake_queue(struct net_device *dev);
void rtl_netif_start_queue(struct net_device *dev);
void rtl_netif_stop_queue(struct net_device *dev);

void rtl_indicate_rx_frame(struct net_device *dev, struct sk_buff *rx_skb, int frame_len);


unsigned long GetCurTime(void);


int rtl_request_timer( timer_t *timer, unsigned long timer_expires, void *timer_func, void *timer_data );
void rtl_delete_timer( timer_t *del_timer_t );
int rtl_check_timer( timer_t *timer );
int rtl_mod_timer(struct timer_list *timer, unsigned long expires);

int rtl8180_open(struct net_device *dev);
int rtl8180_close(struct net_device *dev);
int rtl8180_xmit(struct sk_buff *skb, struct net_device *dev);
struct net_device_stats *rtl8180_get_stats(struct net_device *dev);
int rtl8180_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd);
void rtl8180_set_multicast_list(struct net_device *dev);
void rtl8180_interrupt(int irq, void *dev_id, struct pt_regs *regs);

void rtl8180_init( struct net_device *dev );
void rtl8180_remove( struct net_device *dev );


#endif //#ifndef _INCLUDE_RTL8180_IF_H_
